***************************************************************************;
			*STEP1-1: Read and prepare finance data from Data stream 
***************************************************************************;

libname My 'F:\My_SASdataset';


%macro data_accumulation(conutry);
data date;set &conutry; keep code;run;

proc transpose data=&conutry
               out=australia_transposed;
run;

data e0;set australia_transposed;
if substr(_NAME_,1,1) = '_' then do;

variable=substr(_NAME_,8,2);
*firm=substr(_NAME_,2,5);
firm=substr(_LABEL_,1,6);
end;

if substr(_NAME_,1,4) = 'Code' then do;
	
	variable='datadate';
end;



run; 

 DATA LEADS;
   _N_ ++ _N_ < N;
   SET e0 POINT=_N_;
    firm1 = firm;
   SET e0 NOBS=N END=END;
   IF END THEN firm1 = .;
 RUN;
data e1;set  LEADS;if variable='datadate' then firm = firm1;run;

proc sort data=e1;by firm;run;
proc transpose data=e1
               out=e1_transposed;
by firm;
id variable;
run;
data e1_transposed;set e1_transposed;
format date yymmdd6.; date=datadate;
run;
proc append base=database_timeseries data=e1_transposed force;run;
%mend;


%data_accumulation (my.Australia_timeseries);
%data_accumulation (my.Austria_timeseries);
%data_accumulation (my.Brazil_timeseries);
%data_accumulation (my.Canada_timeseries);
%data_accumulation (my.Chile_timeseries);
%data_accumulation (my.Colombia_timeseries);
%data_accumulation (my.Denmark_timeseries);
%data_accumulation (my.Finland_timeseries);
%data_accumulation (my.France_timeseries);
%data_accumulation (my.Germany_timeseries);
%data_accumulation (my.Italy_timeseries);
%data_accumulation (my.Korea_timeseries);
%data_accumulation (my.Mexico_timeseries);
%data_accumulation (my.Norway_timeseries);
%data_accumulation (my.Portugal_timeseries);
%data_accumulation (my.SouthAfrica_timeseries);
%data_accumulation (my.Sweden_timeseries);
%data_accumulation (my.Switzerland_timeseries);
%data_accumulation (my.UK_timeseries);
%data_accumulation (my.UnitedStates_timeseries);


data my.database_timeseries;set database_timeseries;run;


***************************************************************************;
			*STEP1-2: Read and prepare turnover data from Bloomberg
***************************************************************************;

proc transpose data= MY.TURNOVER_BLOOMBERG   
               out=australia_transposed;
run;

data e0;set australia_transposed;

if substr(_NAME_,1,4) ne 'Date' then do;
found = indexc(_NAME_, "Equity");
bloomberg_firm=substr(_NAME_,1,found-2);
variable=substr(_NAME_,found+7,6);
end;


if substr(_NAME_,1,4) eq 'Date' then do;
	
	variable='datadate';
end;



run; /*** a more efficient and elegant solution by Paul Dorfman  ***/

 DATA LEADS;
   _N_ ++ _N_ < N;
   SET e0 POINT=_N_;
    firm1 = firm;
   SET e0 NOBS=N END=END;
   IF END THEN firm1 = .;
 RUN;
data e1;set  LEADS;if variable='datadate' then firm = firm1;run;

proc sort data=e1;by firm;run;
proc transpose data=e1
               out=e1_transposed;
by firm;
id variable;
run;
data database_timeseries ;set e1_transposed;
format datadate1 yymmdd6.; datadate1=datadate;

run;
data database_timeseries;set database_timeseries ;where firm ne '';run;
proc sort data=database_timeseries ;by firm datadate1;run;

data my.database_timeseries_TURNOVER;set database_timeseries;run;

***************************************************************************;
			*STEP1-3: Read and prepare exchange rate data 
***************************************************************************;


proc transpose data=MY.exchange_rate
               out=rate_transposed;
run;
proc print data=rate_transposed ;run;

data e0;set rate_transposed;
if substr(_LABEL_,1,2) = 'EX' then do;

curr=COMPRESS(substr(_LABEL_,3,2));
variable=COMPRESS(substr(_LABEL_,1,2));
end;

if substr(_NAME_,1,4) = 'Code' then do;
	
	variable='datadate';
end;



run;  /*** a more efficient and elegant solution by Paul Dorfman  ***/

 DATA LEADS;
   _N_ ++ _N_ < N;
   SET e0 POINT=_N_;
    curr1 = curr;
   SET e0 NOBS=N END=END;
   IF END THEN curr1 = .;
 RUN;
data e1;set  LEADS;if variable='datadate' then curr = curr1;run;

proc sort data=e1;by curr;run;
proc transpose data=e1
               out=e1_out;
by curr;
id  variable;
run;

data My.exchange_rate;set e1_out;
format datadate1 yymmdd6.; datadate1=datadate;
run;
***************************************************************************;
			*STEP2: Merge with the dual-class data set
***************************************************************************;

data e0;set my.database_timeseries;
length dscode $ 12;
  dscode=left(firm);

run;
*Merge with the dual-class data set, which is an extended version of the Doidge (2013)'s list;
*
Data strucure:

bdate - Datastream begin date
time - a date variable indicating the end of the dual-class structure
dscode - Datastream identifier
high - equals 1 for high voting class
low - equals 1 for low voting class
vote/share - The number of votes per share
curr - currency
pair - The pair ID for low and high voting class shares

;
data Dual_List;set my.Dual_List;
length dscode1 $ 12;
  dscode1=left(dscode);
  drop dscode;
run;
data Dual_List;set Dual_List;dscode=dscode1;
length bloomberg_firm1 $ 12;
bloomberg_firm1=upcase(bloomberg);

format datadate1 yymmdd6.;
datadate1=time;
drop dscode1;run;





data Dual_List0;set Dual_List;if high=1;run;
data Dual_List1;set Dual_List;if low=1;run;
proc sql;
create table Dual_List00 as 
select t1.*, t2.*  
from e0 as t1, Dual_List0 as t2 
where t1.dscode=t2.dscode ;
quit;
data TURNOVER ;set my.database_timeseries_TURNOVER ;
length bloomberg_firm1 $ 12;
bloomberg_firm1=bloomberg_firm;
format datadate1 yymmdd6.;
datadate1=datadate;
run;
proc sort data=Dual_List00;by bloomberg_firm1 datadate1;run;
proc sort data=TURNOVER;by bloomberg_firm1 datadate1;run;
data Dual_List0000;merge Dual_List00 (in=a) TURNOVER (in=b);by bloomberg_firm1 datadate1;if a;run;

data Dual_List000;set Dual_List0000;
rename name=nameHIGH;
rename high = highHIGH;
rename low = lowHIGH;
rename vote_share = vote_shareHIGH;
rename curr = currHIGH;
rename F10 = countryHIGH;
rename firm = firmHIGH;
rename datadate1 = dateLOW;
rename P_ = PHIGH;
rename dy = dyHIGH;
rename mv = mvHIGH;
rename vo = voHIGH;
rename no = noHIGH;
rename curr = currHIGH;
rename volume=volumeHIGH;
run;
proc sql;create table Dual_List11 as select t1.*, t2.*  from e0 as t1, Dual_List1 as t2 where t1.dscode=t2.dscode;quit;

proc sort data=Dual_List11;by bloomberg_firm1 datadate1;run;
proc sort data=TURNOVER;by bloomberg_firm1 datadate1;run;
data Dual_List1111;merge Dual_List11 (in=a) TURNOVER (in=b);by bloomberg_firm1 datadate1;if a;run;
data Dual_List111;set Dual_List1111;
rename name=nameLOW;
rename LOW = LOWLOW;
rename low = lowLOW;
rename vote_share = vote_shareLOW;
rename curr = currLOW;
rename F10 = countryLOW;
rename firm = firmLOW;
rename datadate1 = dateLOW;
rename P_ = PLOW;
rename dy = dyLOW;
rename mv= mvLOW;
rename vo = voLOW;
rename no = noLOW;
rename curr = currLOW;
rename volume=volumeLOW;
run;


proc sort data=Dual_List000 ;by pair dateLOW;run;
proc sort data=Dual_List111 ;by pair dateLOW;run;
data Dual_List2;merge Dual_List000 Dual_List111;by pair dateLOW;run;
proc sort data=Dual_List2 nodupkey out=Dual_List3 ;by firmlow dateLOW;run;


proc sort data=Dual_List3  ;by firmLOW datadate ;run;
data e1;set Dual_List3 ;
format datadate1 yymmdd6.; datadate1=datadate;
if currLOW eq 'A$' then curr0=1;
if currLOW eq 'C' then curr0=2;
if currLOW eq 'C$' then curr0=3;
if currLOW eq 'CE' then curr0=4;
if currLOW eq 'CP' then curr0=5;
if currLOW eq 'DK' then curr0=6;
if currLOW eq 'DM' then curr0=7;
if currLOW eq 'E' then curr0=8;
if currLOW eq 'KW' then curr0=9;
if currLOW eq 'L' then curr0=10;
if currLOW eq 'MP' then curr0=11;
if currLOW eq 'NK' then curr0=12;
if currLOW eq 'R' then curr0=13;
if currLOW eq 'SF' then curr0=14;
if currLOW eq 'SK' then curr0=15;
if currLOW eq 'U$' then curr0=16;
if currLOW eq 'VB' then curr0=17;
if currLOW eq '' then curr0=18;




run;
data r0;set My.exchange_rate;

if curr eq 'A$' then curr0=1;
if curr eq 'C' then curr0=2;
if curr eq 'C$' then curr0=3;
if curr eq 'CE' then curr0=4;
if curr eq 'CP' then curr0=5;
if curr eq 'DK' then curr0=6;
if curr eq 'DM' then curr0=7;
if curr eq 'E' then curr0=8;
if curr eq 'KW' then curr0=9;
if curr eq 'L' then curr0=10;
if curr eq 'MP' then curr0=11;
if curr eq 'NK' then curr0=12;
if curr eq 'R' then curr0=13;
if curr eq 'SF' then curr0=14;
if curr eq 'SK' then curr0=15;
if curr eq 'U$' then curr0=16;
if curr eq 'VB' then curr0=17;
if curr eq 'P' then curr0=18;

 




run;
*transfer from local currency to USD;
proc sort data=r0;by curr0 datadate1;run;
proc sort data=e1;by curr0 datadate1;run;
data e2;merge r0 (in=a) e1 (in=b);by curr0 datadate1;if a and b;run;



data r1;set e2;
if timelow lt timehigh then time=timelow;
if timelow ge timehigh then time=timehigh;

eyear=year(time);emonth=month(time);eday=day(time);
edate=mdy(emonth,eday,eyear);
calyr=year(datadate1);
calmth=month(datadate1);
calday=day(datadate1);
cdate=mdy(calmth,calday,calyr);
if cdate le edate;
run;


data r2;set r1;

*Compute the WEDGE ************************************************************************;
 superior_shrout=noHIGH;*The number of superior class shares;
 inferior_shrout=  noLOW;*The number of inferior class shares;
 superior_vote=vote_shareHIGH;*The number of votes per superior share;
 inferior_vote=vote_shareLOW;*The number of votes per inferior share;

*Voting Rights = The percentage of total votes held by inferior class shareholders, 
 which is equal to the number of votes per inferior share times the number of inferior shares (inferior votes), 
divided by the sum of inferior votes (as defined above) and superior votes (the number of votes per superior share times the number of superior shares);
VotingRights_I= (inferior_vote*inferior_shrout)/(inferior_vote*inferior_shrout +superior_vote*superior_shrout);

*Cash Flow Rights = The percentage of total cash flow rights held by inferior class shareholders, equal to the number of inferior class shares 
divided by the sum of inferior class and superior class shares. ;
CashflowRights_I=  inferior_shrout/(inferior_shrout+superior_shrout);


*WEDGE = 1- (Voting Rights/Cash Flow Rights);

WEDGE=1-(VotingRights_I/CashflowRights_I);

*Compute the voting premium ***************************************************************;
*rv = the ratio of the number of votes of an inferior voting share to that of a superior
             voting share.;

rv= vote_shareLOW/vote_shareHIGH;

numerator=(PHIGH-PLOW);
denominator=(PLOW-rv*PHIGH);
votingpremium=numerator/denominator ;


*Compute MARKET_CAP ***************************************************************;
mvLOW_USD=mvLOW/ex;
mvHIGH_USD=mvHIGH/ex;
*The natural log of a firms total market capitalization (where total market capitalization is 
the sum of the market capitalization of the superior voting and inferior voting share classes (in millions of US$)).;
MARKET_CAP=log(mvLOW_USD+mvHIGH_USD);



*Fraction voting shares is equal to NSH/(NSH + NSL);
FractionVS=noHIGH/(noHIGH+noLOW);


*The excess dividend payment to a limited- over a multiple-voting share, scaled
by the total dividend to a limited-voting share. ;
Excess_div=(dLOW-dHigh)/dLOW;

*Voting power is the fraction of total votes attached to the high voting share class relative to the
fraction of total cash flow rights attached to the high voting share class.;
votingpower=(noHIGH+noLOW)/(noHIGH+rv*noLOW);

*Relative turnover is the log of the ratio of the total turnover of the low
voting shares relative to the total turnover of the high voting shares.;
turnover=log((voLOW/noLOW)/(voHIGH/noHIGH));


year=year(fiscaldate);month=month(fiscaldate);
fic=countryLOW;

* compute return for the inferior class shares ****************************************************;
*The dividend yield expresses the dividend per share as a percentage of the share price. ;
if firmlow = lag(firmlow) then do;
	PLOW0=lag(PLOW);
	div0=(dyLOW)*PLOW;
	if div0 eq . then div=0;else div=div0;
	retLOW=(PLOW+div-PLOW0)/PLOW0;
end;

run;

data r3;set r2;where year(datadate1) between 1994 and 2007;if pair ne . then dual_flag=1;else dual_flag=0;run;
*collect industry infor from WORLDSCOPE;
proc sql;create table industry as select distinct sedol, sich as dnum from my2.industry_SAMPLE ;quit;
proc sql;
create table  r4 as
select t1.*,t2.*
from r3 as t1 left join industry as t2
on  t1.sedol=t2.sedol;quit;


*Merge with the market returns ****************************************************************************;
data r5;set r4;


country=compress(Upcase(countrylow));
run;
proc sort data=r5;by firmlow datadate1;
*country-level return;
*merge w/ market index data;
*Market return is the return on the Datastream index for the country of listing over the three months 
preceding the offering.;
data market;set My.global_index;
where fic ne '';
country=fic;
market=PRCCD;
datadate1=datadate;
format datadate datadate1 date9.;

lnret=log(mkt_ret+1);
run;
proc sort data=market;by country datadate1;

*We correct for non-synchronous trading by including two lead and
lag terms for the local and U.S. market indexes.;
data market;set market;

day= weekday( datadate1);
if day eq 6;
drop day;
data market_0;set market;
	market0=lag(market);
	retMKT=(market-market0)/market0;
	retMKT_lag=lag(retMKT);
	retMKT_lag2=lag2(retMKT);
run;
proc sort data=market_0;by country datadate1;
data market_0;set market_0;by country;if first.country then delete;run;
proc sort data=market_0;by country datadate1;
data market_0;set market_0;by country;if first.country then delete;run;
proc sort data=market_0 out= market_1;by country descending datadate1;
data market_1;set market_1;

	retMKT_lead=lag(retMKT);
	retMKT_lead2=lag2(retMKT);
run;
proc sort data=market_1;by country descending datadate1;
data market_1;set market_1;by country;if first.country then delete;run;
proc sort data=market_1;by country descending datadate1;
data market_1;set market_1;by country;if first.country then delete;run;


proc sort data=market_1;by country  datadate1;run;

*merge b/w country-level and firm-level data;
proc sort data=r5;by country datadate1;
data r6;merge r5 (in=a) market_1 (in=b);by country datadate1;if a and b;run;

*merge w/ US index return data;
data us0;set My.Us_mktindex;
day=weekday( date);
datadate1=date;
if day eq 6;

 us=SPRTRN;

	us_lag=lag(us);
	us_lag2=lag2(us);
run;
proc sort data=us0 out= us1;by  descending datadate1;
data us1;set us1;

	us_lead=lag(us);
	us_lead2=lag2(us);
	drop day;
run;

*merge b/w country-level and firm-level data;
proc sort data=r6;by  datadate1;
proc sort data=us1;by  datadate1;
data r7;merge r6 (in=a) us1 (in=b);by  datadate1;if a and b;run;


*merge w/ exchange rate data;
data ex0;set My.exchange_rate;
day=weekday( date);
datadate1=date;
if day eq 6;
run;
data ex0;set ex0;
	rate_lag=lag(rate);
	rate_lag2=lag2(rate);
run;
proc sort data=ex0;by  country datadate1;
data ex0;set ex0;by country ;if first.country then delete;
proc sort data=ex0;by  country datadate1;
data ex0;set ex0;by country ;if first.country then delete;run;
proc sort data=ex0 out= ex1;by country descending datadate1;
data ex1;set ex1;

	rate_lead=lag(rate);
	rate_lead2=lag2(rate);
run;
proc sort data=ex1;by country descending datadate1;
data ex1;set ex1;by country ;if first.country then delete;run;
proc sort data=ex1;by country descending datadate1;
data ex1;set ex1;by country ;if first.country then delete;drop day;run;

*merge b/w country-level and firm-level data;
proc sort data=ex1;by country  datadate1;
proc sort data=r7;by country datadate1;
data r8;merge r7 (in=a) ex1 (in=b);by country datadate1;if a and b;run;
proc sort data=r8;by firmlow datadate1;
data r9;
set r8;

*The dividend yield expresses the dividend per share as a percentage of the share price. ;
if firmlow = firmlow0 then do;
	PLOW0=lag(PLOW);
	div0=(dyLOW)*PLOW;
	if div0 eq . then div=0;else div=div0;
	retLOW=(PLOW+div-PLOW0)/PLOW0;
end;


if retLOW ne .;
if retMKT ne .;
run;
data r10;set r9;by firmlow;if first.firmlow then retLOW=.;count=1;run;
*Following MYY, we exclude stocks that trade for less than 26 weeks 
during a particular year.;

proc sort data=r10;by firmlow calyr;
proc means data=r10 noprint;
var count;
by firmlow calyr;
output out=e5_out sum=count_sum;
quit;

data delete;set e5_out;if count_sum lt 26 then delete;
keep firmLOW  calyr;
run;

proc sort data=r10;by firmLOW  calyr;
proc sort data=delete;by firmLOW  calyr;
data e5;merge delete (in=a) r10 (in=b);by firmLOW  calyr;if a and b;run;


*regression model;
option nonotes;
proc sort data=e5;by firmlow calyr;
ods output "Nonlinear GMM Summary of Residual Errors"=fitresult;
proc model data=e5 ; 
by firmlow calyr;
 parms a b1 b2 b3 b4 b5 b6 b7 b8 b9 b10;
 instruments retMKT us rate retMKT_lag us_lag rate_lag retMKT_lag2 us_lag2 rate_lag2 retMKT_lead
us_lead rate_lead rate_lead2 us_lead2 rate_lead2;
 retLOW =a+b1*retMKT + b2*(us + rate) + b3*retMKT_lag +b4*(us_lag + rate_lag) 
+ b5*retMKT_lag2 + b6*(us_lag2+rate_lag2) + b7*retMKT_lead + b8*(us_lead+rate_lead) + b9*rate_lead2
+b10*(us_lead2 + rate_lead2); 
 fit retLOW / gmm kernel=(bart, 3, 0);
 ods output parameterestimates=param1  fitstatistics=fitresult
 OutputStatistics=residual;
 quit;
 ods output close;

 option notes;
 data a b1 b2 b3 b4 b5 b6 b7 b8 b9 b10;set param1;
 if  Parameter eq 'a' then output a;
 if  Parameter eq 'b1' then output b1;
 if  Parameter eq 'b2' then output b2;
 if  Parameter eq 'b3' then output b3;
 if  Parameter eq 'b4' then output b4;
 if  Parameter eq 'b5' then output b5;
 if  Parameter eq 'b6' then output b6;
 if  Parameter eq 'b7' then output b7;
 if  Parameter eq 'b8' then output b8;
 if  Parameter eq 'b9' then output b9;
 if  Parameter eq 'b10' then output b10;
 run;

data a; set a;
a_est=Estimate;
keep firmLOW  calyr  a_est;
data b1; set b1;
b1_est=Estimate;
keep firmLOW  calyr  b1_est;

data b2; set b2;
b2_est=Estimate;
keep firmLOW  calyr  b2_est;

data b3; set b3;
b3_est=Estimate;
keep firmLOW  calyr  b3_est;

data b4; set b4;
b4_est=Estimate;
keep firmLOW  calyr  b4_est;


data b5; set b5;
b5_est=Estimate;
keep firmLOW  calyr  b5_est;
data b6; set b6;
b6_est=Estimate;
keep firmLOW  calyr  b6_est;

data b7; set b7;
b7_est=Estimate;
keep firmLOW  calyr  b7_est;

data b8; set b8;
b8_est=Estimate;
keep firmLOW  calyr  b8_est;

data b9; set b9;
b9_est=Estimate;
keep firmLOW  calyr  b9_est;

data b10; set b10;
b10_est=Estimate;
keep firmLOW  calyr  b10_est;





proc sort data=a;by firmLOW  calyr;
proc sort data=b1;by firmLOW  calyr;
data e5_2;merge b1 (in=a) a (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_2;by firmLOW  calyr;
proc sort data=b2;by firmLOW  calyr;
data e5_3;merge b2 (in=a) e5_2 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_3;by firmLOW  calyr;
proc sort data=b3;by firmLOW  calyr;
data e5_4;merge b3 (in=a) e5_3 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_4;by firmLOW  calyr;
proc sort data=b4;by firmLOW  calyr;
data e5_5;merge b4 (in=a) e5_4 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_5;by firmLOW  calyr;
proc sort data=b5;by firmLOW  calyr;
data e5_6;merge b5 (in=a) e5_5 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_6;by firmLOW  calyr;
proc sort data=b6;by firmLOW  calyr;
data e5_7;merge b6 (in=a) e5_6 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_7;by firmLOW  calyr;
proc sort data=b7;by firmLOW  calyr;
data e5_8;merge b7 (in=a) e5_7 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_8;by firmLOW  calyr;
proc sort data=b8;by firmLOW  calyr;
data e5_9;merge b8 (in=a) e5_8 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_9;by firmLOW  calyr;
proc sort data=b9;by firmLOW  calyr;
data e5_10;merge b9 (in=a) e5_9 (in=b);by firmLOW  calyr;if a and b;run;


proc sort data=e5_10;by firmLOW  calyr;
proc sort data=b10;by firmLOW  calyr;
data e5_11;merge b10 (in=a) e5_10 (in=b);by firmLOW  calyr;if a and b;run;

proc sort data=e5;by firmLOW  calyr;
proc sort data=e5_11;by firmLOW  calyr;
data e5_12;merge e5_11 (in=a) e5 (in=b);by firmLOW  calyr;if a and b;run;
* compute Firm-Specific- Weekly Return *****************************************************************************;
data e6;set e5_12;
resid0= retLOW -(a_est+b1_est*retMKT + b2_est*(us + rate) + b3_est*retMKT_lag +b4_est*(us_lag + rate_lag) 
+ b5_est*retMKT_lag2 + b6_est*(us_lag2+rate_lag2) + b7_est*retMKT_lead + b8_est*(us_lead+rate_lead) + b9_est*rate_lead2
+b10_est*(us_lead2 + rate_lead2));
resid=log(1+resid0);
resid2=resid*resid;
resid3=resid*resid*resid;
run;

*compute NCSKEW variable *****************************************************************************;
proc sort data=e6;by firmLOW calyr;run;
proc means data=e6 mean  n noprint;
var resid resid2 resid3;
by firmLOW calyr;
output out=NCSKEW_out mean=resid_mean resid2_mean resid3_mean
sum= resid_sum resid2_sum resid3_sum
n=n resid2_n resid3_n
skew=resid_skew;
run;

proc sort data=NCSKEW_out;by firmLOW calyr;run;
data NCSKEW;set NCSKEW_out;
NCSKEW=(-1)*((n*((n-1)**(3/2))*resid3_sum)/((n-1)*(n-2)*((resid2_sum)**(3/2)))) ;
CSKEW= lag(NCSKEW);

*compute DUVOL variable *****************************************************************************;
*For each firm i over a fiscal year period t, we separate all the weeks with firm-specific
weekly returns, Wi,??, below the annual mean (down weeks) from those with
firm-specific returns above the annual mean (up weeks) and calculate the standard
deviation for each of these subsamples separately. We then measure DUVOL as the
natural logarithm of the ratio of the standard deviation in the down weeks to the standard
deviation in the up weeks.;
proc sort data=e6;by firmLOW  calyr;
proc sort data=NCSKEW;by firmLOW  calyr;
data e7;merge NCSKEW (in=a) e6 (in=b);by firmLOW  calyr;if a and b;run;

data down up;set e7;
if resid lt resid_mean then output down;
if resid gt resid_mean then output up;

run;
proc sort data=down;by firmLOW calyr ;run;
proc means data=down mean  n noprint;
var resid ;
output out=DUVOL_down std=resid_D;
by firmLOW calyr ;
run;

proc sort data=up;by firmLOW calyr ;run;
proc means data=up mean  n noprint;
var resid ;
output out=DUVOL_up std=resid_U;
by firmLOW calyr ;
run;
*merge b/w the up and the down data;
proc sort data=DUVOL_up;by firmLOW calyr;
proc sort data=DUVOL_down;by firmLOW calyr;
data DUVOL;merge DUVOL_up (in=a) DUVOL_down (in=b);by firmLOW calyr;if a and b;run;

data DUVOL;set DUVOL;DUVOL=log(resid_D/resid_U) ;run;


*compute CRASH variable *****************************************************************************;
*This paper defines crash weeks in a given fiscal year for a given firm as those
weeks during which the firm experiences firm-specific
weekly returns 3.0 standard deviations below the mean
firm-specific weekly returns over the entire fiscal year.
The first measure of crash likelihood for each firm in each
year, denotedby CRASH, is an indicator variable that
equals one for a firm-year that experiences one or more
crash weeks(as defined above) during the fiscal-year
period, and zero otherwise.
;
proc sort data=e6;by firmLOW calyr;run;
proc means data=e6 mean  n noprint;
var resid ;
by firmLOW calyr;
output out=CRASH0_out mean=resid_mean 
std= resid_std
n= resid_n ;
run;
*merge b/w the original data;
proc sort data=e6;by firmLOW calyr;
proc sort data=CRASH0_out;by firmLOW calyr;
data CRASH0;merge e6 (in=a) CRASH0_out (in=b);by firmLOW calyr;if a and b;run;
data CRASH0;set CRASH0;
criteria = resid_mean-3*resid_std;
if resid lt criteria then n_crash0=1;else n_crash0=0;

run;

proc sort data=CRASH0;by firmLOW calyr;
proc means data=CRASH0 mean  n noprint;
var n_crash0 ;
by firmLOW calyr;
output out=CRASH_out sum=n_crash0_sum 
n= crash0_n ;
run;

data CRASH;set CRASH_out;
if n_crash0_sum ge 1 then crash =1;else crash=0;
keep firmLOW calyr crash ; 
run;


*Averaging each variable annually;

proc sort data=e6;by country firmLOW calyr;run;
proc means data=e6 mean  n noprint;
var retLOW WEDGE dLOW dHigh FractionVS Excess_div numerator denominator votingpremium  MKTCAP dyHIGH dyLOW votingpower turnover;
by country firmLOW calyr;
output out=c_out mean=retLOW WEDGE dLOW_avg dHigh_avg FractionVS_avg Excess_div_avg numerator_avg denominator_avg votingpremium_avg  MKTCAP dyHIGH_avg dyLOW_avg votingpower_avg turnover_avg
std= retLOW_std;
run;
data c_out1;set c_out;
calyr=calyr;


*DTURN is the average monthly share turnover over the current year, minus the average monthly
share turnover over the previous year, where monthly share turnover is calculated as the monthly
trading volume divided by total number of shares outstanding during the month.;
firmLOW0=lag(firmLOW);
if firmLOW eq firmLOW0 then turnover_avg0=lag(turnover_avg);
DTURN=turnover_avg/turnover_avg0;


*SIGMA is the standard deviation of the Firm-Specific-Weekly Return over the current year.;
SIGMA= retLOW_std;
* RET is the mean of the Firm-Specific-Weekly Return over the current year.;
RET= retLOW;


run;
*If DTURN is missing, replace it w/ the sample median in the same country and year;
proc sort data=c_out1;by country calyr;
proc means data=c_out1 noprint;
var DTURN;
by country calyr;
output out=c_out2 median=DTURN_median;
quit;
*merge back to the annual average data;
proc sort data=c_out1;by country calyr;
proc sort data=c_out2;by country calyr;
data c_out3;merge c_out1 (in=a) c_out2 (in=b);by country calyr;if a and b;
if missing(DTURN) then DTURN=DTURN_median;
run;


*merge back to the voting premium data;

proc sort data=c_out3;by firmLOW calyr;
proc sort data=fitresult;by firmLOW calyr;
data e7;merge c_out3 (in=a) fitresult (in=b);by firmLOW calyr;if a and b;run;*4,532;


proc sort data=e7;by firmLOW calyr;
proc sort data=NCSKEW;by firmLOW calyr;
data e7;merge e7 (in=a) NCSKEW (in=b);by firmLOW calyr;if a and b;run;*4,532;

proc sort data=e7;by firmLOW calyr;
proc sort data=DUVOL;by firmLOW calyr;
data e8;merge e7 (in=a) DUVOL (in=b);by firmLOW calyr;if a and b;run;*4,481;

proc sort data=e8;by firmLOW calyr;
proc sort data=CRASH;by firmLOW calyr;
data e9;merge e8 (in=a) CRASH (in=b);by firmLOW calyr;if a and b;run;*4,481;

*Construct control variables for te multivariate analysis ********************************************;

*merge w/ ret data;
data e10;set e9;year1=calyr;

data high;set my.Dual_List;
dscode1=dscode*1;
where high = 1;
rename name=nameHIGH;
rename high = highHIGH;
rename low = lowHIGH;
rename vote_share = vote_shareHIGH;
rename curr = currHIGH;
rename F10 = countryHIGH;
rename dscode = firmHIGH;
rename dscode1 = dscode_HIGH;


data low;set my.Dual_List;
dscode1=dscode*1;
where low = 1;
rename name=nameLOW;
rename high = highLOW;
rename low = lowLOW;
rename vote_share = vote_shareLOW;
rename curr = currLOW;
rename F10 = countryLOW;
rename dscode = firmLOW;
rename dscode1 = dscode_low;

run;
proc sql;create table dual0 as select * from low as a , high as b where a.pair=b.pair ;quit;




proc sql;create table comb0 as select * from e10 as a , dual0 as b where a.firmlow=b.firmLOW ;quit;

*add SEDOL code from WORLDSCOPE;
proc sql;create table comb0_low as select * from comb0 as a , My.sedol as b where a.dscode_low=b.type;quit;
proc sql;create table comb0_high as select * from comb0 as a , My.sedol as b where a.dscode_HIGH=b.type;quit;



*Merge with accoutning data from WORLDSCOPE;
data WORLDSCOPE_data;set My.WORLDSCOPE_data;
if missing(sedol) then delete;



run;

proc sql;create table comb_low as select * from comb0_low as a , WORLDSCOPE_data as b where a.sedol_code=b.sedol and a.year1=b.year;quit;
proc sql;create table comb_high as select * from comb0_high as a , WORLDSCOPE_data as b where a.sedol_code=b.sedol and a.year1=b.year;quit;


data comb1;set comb_low comb_high;run;


proc sort data=comb1 nodupkey out=comb2;by sedol year;run;


data comb2;set comb2;

AT=toasset;
AT_lag=lag(AT);

*MB	=	The ratio of market value of equity (where market value of equity is the sum of the market value of the superior voting and inferior voting share classes) 
to the book value of equity at the end of the year.
LEVERAGE	=	The book value of long-term debt scaled by the sum of market value of equity and book value of long-term debt at the end of the year.
ROA	=	Income before extraordinary items divided by the beginning-of-year total assets.
;
ROA= NIBE/AT_lag; 
MB= MKTCAP/toequity;
LEV=Longterm_debt/(Longterm_debt+MKTCAP) ;

*OPER CYCLE	=	The log of receivables to sales plus inventory to COGS multiplied by 360. ;
OperCycle=log(((it02051/it01001) + (it02101/it01051))*360);
*STD CFO	=	The firm-specific standard deviation of the ratio of cash flow from operations and average total assets from years t-5 to t-1;
CFO = it04201;
*CFO/SALES	=	The ratio of cash flow from operating activities (CFO) to sales.;
CFOsale = it04201/it01001;
run;


proc sort data=comb2; by sedol year ;
data d10; set comb2;
by sedol year ; 

datadate=mdy(1,1,year);
beg=intnx('year',datadate, -1);
end12=intnx('year',datadate, -5);
format beg end12 datadate yymmdd.;

beg_year=year(beg);
beg_month=month(beg);
end12_year=year(end12);
end12_month=month(end12);
keep sedol    beg beg_year beg_month end12 end12_year end12_month;
run;

proc sort data=d10; by sedol end12  beg;
proc sort data=d9; by sedol year;
proc sql;
  create table d11_0 as select
  d9.year,  d10.beg, d10.beg_year, d10.beg_month, d10.end12, d10.end12_year, d10.end12_month,
d9.sedol, d9.year,d9.CFO 
  from d10, comb2 as d9
  where d9.sedol=d10.sedol and d9.year between d10.end12_year  and  d10.beg_year ;
quit;
proc sort data=d11_0 ;by sedol beg_year;
proc means data=d11_0 noprint;
var CFO;
by sedol beg_year;
output out=o0_out std=StdCFO;
quit;
proc sql;
create table comb3 as
select b.*,a.StdCFO as StdCFO_lag
from o0_int a, comb2 b
where a.sedol=b.sedol and  a.beg_year=b.year;
quit;
*merge w/ smoothing data ****************************************************;



data opaque1;set My.WS_EXT_INT_SMTH;
where it06006 ne '' and IT02999 ne .;
sedol=it06006;
sedol_lag=lag(sedol);

if IT04851 ne . then c_CH=IT04851;else  c_CH=dif(IT02001);
c_TP0=dif(IT03063);
if c_TP0=. then c_TP=0;else c_TP=c_TP0;
c_STD0=dif(IT03051);
if c_STD0=. then c_STD=0;else c_STD=c_STD0;
CurrentAssets_Total =IT02201;
CurrentLiabilities_Total=IT03101;
Depreciation=IT01151;
accruals= dif(CurrentAssets_Total) - dif(CurrentLiabilities_Total) -c_CH +c_STD +c_TP -Depreciation;

operatingIncome=IT01250;

ocf= operatingIncome - accruals;



*OPAQUE	=	The correlation between changes in  cash flow from operations divided by total assets and total accruals 
scaled by total assets during the previous five years rolling window (Leuz et al. 2003). Cash ?ow from operations is 
equal to operating income minus accruals, where accruals are calculated as: 
(?total current assets  ?cash)-(?total current liabilities - ?short-term debt - ?taxes payable) - depreciation expense.
This correlation is multiplied by negative one such that firms with larger OPAQUE values correspond to those with more smooth earnings.;


SMTH_numerator= dif(ocf);*/lag(IT02999);
SMTH_denominator= dif(accruals);*/lag(IT02999);


run;

proc sort data=opaque1;by sedol year;
data opaque1;set opaque1;by sedol ;if first.sedol then delete;

run;
proc sort data=opaque1;by sedol year;
data d9;set opaque1;by sedol ;if first.sedol then delete;

run;

proc sort data=d9; by sedol year ;
data d10; set d9;
by sedol year ; 

datadate=mdy(1,1,year);
beg=intnx('year',datadate, -1);
end12=intnx('year',datadate, -5);
format beg end12 datadate yymmdd.;

beg_year=year(beg);
beg_month=month(beg);
end12_year=year(end12);
end12_month=month(end12);
keep sedol    beg beg_year beg_month end12 end12_year end12_month;
run;

proc sort data=d10; by sedol end12  beg;
proc sort data=d9; by sedol year;
proc sql;
  create table d11_0 as select
  d9.year,  d10.beg, d10.beg_year, d10.beg_month, d10.end12, d10.end12_year, d10.end12_month,
d9.sedol, d9.year,d9.SMTH_numerator, d9.SMTH_denominator
  from d10, d9
  where d9.sedol=d10.sedol and d9.year between d10.end12_year  and  d10.beg_year ;
quit;

 option nonotes;
proc sort data=d11_0; by sedol beg_year ;
proc corr data=d11_0 noprint spearman outp=CorrOutp;
var SMTH_numerator SMTH_denominator  ;
by sedol beg_year;
quit;

data CorrOutp1;set CorrOutp;

if _TYPE_ ="CORR" and _NAME_="SMTH2_denominator" then smoothing=SMTH2_numerator*(-1);

keep sedol beg_year smoothing ;
run;
data smoothing;set CorrOutp1;
where smoothing ne .;
keep sedol beg_year  smoothing;
run;

data number;set CorrOutp;
where _TYPE_ ="N" ;
number1=SMTH_numerator;
number2=SMTH_denominator;
*if number1=5 and number2=5;
if (3 le number1 le 5) and (3 le number2 le 5) ;
keep sedol beg_year number1 ;
run;
proc sql;
  create table smoothing_DT as select
  s2.*, d12.beg_year as year
  from smoothing as s2 , number as d12 
  where s2.sedol=d12.sedol and s2.beg_year = d12.beg_year;
quit;


proc sql;
  create table comb4 as select
  d1.*, d2.smoothing as smoothing_lag
  from comb3 as d1 , smoothing_DT as d2 
  where d1.sedol=d2.sedol and d1.year = d2.beg_year;
quit;



proc sort data=comb4 nodupkey out=comb5;by firmlow calyr;run;
data comb5;set comb5;
if firmlow=lag(firmlow) then do;
IC_lag =lag(IC);
CSKEW_lag=lag(NCSKEW);
DTURN_lag=lag(DTURN);
SIGMA_lag=lag(SIGMA);
RET_lag=lag(RET);
MKTCAP_lag=lag(MKTCAP);
MB_lag=lag(MB);
LEV_lag=lag(LEV);
ROA_lag=lag(ROA);
OperCycle_lag=lag(OperCycle);
CFOsale_lag=lag(CFOsale); 
end;
run;
proc sort data=comb5 ;by firmlow calyr;run;
data comb5;set comb5;by firmlow;if first.firmlow then delete;

proc sort data=comb5;by country year ;
proc rank data=comb5 groups=2 out=comb6;
var smoothing_lag;
ranks smoothing_ranked;
by country year  ;
quit;

data low0 high0;set comb6;
if smoothing_ranked in (0) then output low0;
if smoothing_ranked in (1) then output high0;
run;
*Winsorization ************************************************************************************;

/**********************************************************************************************/
/* FILENAME:        Winsorize_Truncate.sas                                                    */
/* ORIGINAL AUTHOR: Steve Stubben (Stanford University)                                       */
/* MODIFIED BY:     Ryan Ball (UNC-Chapel Hill)                                               */
/* DATE CREATED:    August 3, 2005                                                            */
/* LAST MODIFIED:   August 3, 2005                                                            */
/* MACRO NAME:      Winsorize_Truncate                                                        */
/* ARGUMENTS:       1) DSETIN: input dataset containing variables that will be win/trunc.     */
/*                  2) DSETOUT: output dataset (leave blank to overwrite DSETIN)              */
/*                  3) BYVAR: variable(s) used to form groups (leave blank for total sample)  */
/*                  4) VARS: variable(s) that will be winsorized/truncated                    */
/*                  5) TYPE: = W to winsorize and = T (or anything else) to truncate          */
/*                  6) PCTL = percentile points (in ascending order) to truncate/winsorize    */
/*                            values.  Default is 1st and 99th percentiles.                   */
/* DESCRIPTION:     This macro is capable of both truncating and winsorizing one or multiple  */
/*                  variables.  Truncated values are replaced with a missing observation      */
/*                  rather than deleting the observation.  This gives the user more control   */
/*                  over the resulting dataset.                                               */
/* EXAMPLE(S):      1) %Winsorize_Truncate(dsetin = mydata, dsetout = mydata2, byvar = year,  */
/*                          vars = assets earnings, type = W, pctl = 0 98)                    */
/*                      ==> Winsorizes by year at 98% and puts resulting dataset into mydata2 */
/**********************************************************************************************/

%macro Winsorize_Truncate(dsetin = , 
                          dsetout = , 
                          byvar = none, 
                          vars = , 
                          type = W, 
                          pctl = 1 99);

    %if &dsetout = %then %let dsetout = &dsetin;
    
    %let varL=;
    %let varH=;
    %let xn=1;

    %do %until (%scan(&vars,&xn)= );
        %let token = %scan(&vars,&xn);
        %let varL = &varL &token.L;
        %let varH = &varH &token.H;
        %let xn = %EVAL(&xn + 1);
    %end;

    %let xn = %eval(&xn-1);

    data xtemp;
        set &dsetin;

    %let dropvar = ;
    %if &byvar = none %then %do;
        data xtemp;
            set xtemp;
            xbyvar = 1;

        %let byvar = xbyvar;
        %let dropvar = xbyvar;
    %end;

    proc sort data = xtemp;
        by &byvar;

    /*compute percentage cutoff values*/
    proc univariate data = xtemp noprint;
        by &byvar;
        var &vars;
        output out = xtemp_pctl PCTLPTS = &pctl PCTLPRE = &vars PCTLNAME = L H;

    data &dsetout;
        merge xtemp xtemp_pctl; /*merge percentage cutoff values into main dataset*/
        by &byvar;
        array trimvars{&xn} &vars;
        array trimvarl{&xn} &varL;
        array trimvarh{&xn} &varH;

        do xi = 1 to dim(trimvars);
            /*winsorize variables*/
            %if &type = W %then %do;
                if trimvars{xi} ne . then do;
                    if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = trimvarl{xi};
                    if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = trimvarh{xi};
                end;
            %end;
            /*truncate variables*/
            %else %do;
                if trimvars{xi} ne . then do;
                    /*insert .T code if value is truncated*/
                    if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = .T;
                    if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = .T;
                end;
            %end;
        end;
        drop &varL &varH &dropvar xi;

    /*delete temporary datasets created during macro execution*/
    proc datasets library=work nolist;
        delete xtemp xtemp_pctl; quit; run;

%mend;
*Creates the final subset with lower opacity;
%Winsorize_Truncate(dsetin =low0 ,
                            dsetout = my.low,
                            byvar =  none,
                            vars = NCSKEW DUVOL

    smoothing_lag
  CSKEW_lag
  DTURN_lag
  SIGMA_lag
  RET_lag
  MKTCAP_lag
  MB_lag
  LEV_lag
  ROA_lag OperCycle_lag CFOsale_lag StdCFO_lag,
                            type = w,
                            pctl =   1 99);


*Creates the final subset with higher opacity;
%Winsorize_Truncate(dsetin =high0 ,
                            dsetout = my.high,
                            byvar =  none,
                            vars = NCSKEW DUVOL

  smoothing_lag
  CSKEW_lag
  DTURN_lag
  SIGMA_lag
  RET_lag
  MKTCAP_lag
  MB_lag
  LEV_lag
  ROA_lag OperCycle_lag CFOsale_lag StdCFO_lag,
                            type = w,
                            pctl =   1 99);
